@@ -1,23 +1,8 @@
 /*
+ * Embedded Linux library
+ * Copyright (C) 2011-2014  Intel Corporation
  *
- *  Embedded Linux library
- *
- *  Copyright (C) 2011-2014  Intel Corporation. All rights reserved.
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #ifdef HAVE_CONFIG_H
@@ -39,6 +24,7 @@
 #include "dbus-private.h"
 #include "string.h"
 #include "queue.h"
+#include "utf8.h"
 
 #define DBUS_MAX_INTERFACE_LEN 255
 #define DBUS_MAX_METHOD_LEN 255
@@ -341,9 +327,18 @@ bool _dbus_parse_unique_name(const char *name, uint64_t *out_id)
 	if (!l_str_has_prefix(name, ":1."))
 		return false;
 
+	name += 3;
+
+	/*
+	 * Disallow '+' or '-' at the beginning of the string
+	 * must have at least one digit
+	 */
+	if (!l_ascii_isdigit(*name))
+		return false;
+
 	errno = 0;
-	r = strtoull(name + 3, &endp, 10);
-	if (!endp || endp == name || *endp || errno)
+	r = strtoull(name, &endp, 10);
+	if (!endp || *endp || errno)
 		return false;
 
 	if (out_id)
@@ -706,7 +701,7 @@ bool _dbus1_iter_get_fixed_array(struct l_dbus_message_iter *iter,
 	size = get_basic_size(type);
 
 	/* Fail if the array is not a fixed size or contains file descriptors */
-	if (!size || type == 'n')
+	if (!size || type == 'h')
 		return false;
 
 	/*
