@@ -22,7 +22,7 @@
 
 #include <glib.h>
 
-#include "lib/bluetooth.h"
+#include "bluetooth/bluetooth.h"
 #include "bluetooth/l2cap.h"
 #include "btio/btio.h"
 #include "src/log.h"
@@ -2416,7 +2416,7 @@ static gboolean read_btclock(struct mcap_mcl *mcl, uint32_t *btclock,
 	/*
 	 * FIXME: btd_adapter_read_clock(...) always return FALSE, current
 	 * code doesn't support CSP (Clock Synchronization Protocol). To avoid
-	 * build dependancy on struct 'btd_adapter', removing this code.
+	 * build dependency on struct 'btd_adapter', removing this code.
 	 */
 
 	return FALSE;
@@ -2506,7 +2506,7 @@ static gboolean initialize_caps(struct mcap_mcl *mcl)
 
 	_caps.ts_acc = 20; /* ppm, estimated */
 
-	/* A little exercise before measuing latency */
+	/* A little exercise before measuring latency */
 	clock_gettime(CLK, &t1);
 	read_btclock_retry(mcl, &btclock, &btaccuracy);
 
@@ -2878,7 +2878,7 @@ static void proc_sync_set_req(struct mcap_mcl *mcl, uint8_t *cmd, uint32_t len)
 			return;
 		}
 
-		/* Convert to miliseconds */
+		/* Convert to milliseconds */
 		phase2_delay = bt2ms(phase2_delay);
 
 		if (phase2_delay > 61*1000) {
@@ -2961,12 +2961,12 @@ static void proc_sync_cap_rsp(struct mcap_mcl *mcl, uint8_t *cmd, uint32_t len)
 	gpointer user_data;
 
 	if (mcl->csp->csp_req != MCAP_MD_SYNC_CAP_REQ) {
-		DBG("CSP: got unexpected cap respose");
+		DBG("CSP: got unexpected cap response");
 		return;
 	}
 
 	if (!mcl->csp->csp_priv_data) {
-		DBG("CSP: no priv data for cap respose");
+		DBG("CSP: no priv data for cap response");
 		return;
 	}
 
@@ -2979,7 +2979,7 @@ static void proc_sync_cap_rsp(struct mcap_mcl *mcl, uint8_t *cmd, uint32_t len)
 	mcl->csp->csp_req = 0;
 
 	if (len != sizeof(mcap_md_sync_cap_rsp)) {
-		DBG("CSP: got corrupted cap respose");
+		DBG("CSP: got corrupted cap response");
 		return;
 	}
 
@@ -3009,12 +3009,12 @@ static void proc_sync_set_rsp(struct mcap_mcl *mcl, uint8_t *cmd, uint32_t len)
 	gpointer user_data;
 
 	if (mcl->csp->csp_req != MCAP_MD_SYNC_SET_REQ) {
-		DBG("CSP: got unexpected set respose");
+		DBG("CSP: got unexpected set response");
 		return;
 	}
 
 	if (!mcl->csp->csp_priv_data) {
-		DBG("CSP: no priv data for set respose");
+		DBG("CSP: no priv data for set response");
 		return;
 	}
 
@@ -3027,7 +3027,7 @@ static void proc_sync_set_rsp(struct mcap_mcl *mcl, uint8_t *cmd, uint32_t len)
 	mcl->csp->csp_req = 0;
 
 	if (len != sizeof(mcap_md_sync_set_rsp)) {
-		DBG("CSP: got corrupted set respose");
+		DBG("CSP: got corrupted set response");
 		return;
 	}
 
