@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  *
  *  BlueZ - Bluetooth protocol stack for Linux
@@ -8,38 +9,21 @@
  *  Copyright (C) 2002-2003  Stephen Crane <steve.crane@rococosoft.com>
  *
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <stdio.h>
-#include <errno.h>
 #include <stdlib.h>
-#include <sys/socket.h>
+#include <stdbool.h>
 
-#include <bluetooth/bluetooth.h>
-#include <bluetooth/l2cap.h>
-#include <bluetooth/sdp.h>
-#include <bluetooth/sdp_lib.h>
+#include "bluetooth/bluetooth.h"
+#include "bluetooth/sdp.h"
+#include "bluetooth/sdp_lib.h"
 
 #include "sdpd.h"
-#include "logging.h"
+#include "log.h"
 
 static sdp_list_t *service_db;
 static sdp_list_t *access_db;
@@ -54,10 +38,10 @@ typedef struct {
  * The service repository is a linked list in sorted order
  * and the service record handle is the sort key
  */
-static int record_sort(const void *r1, const void *r2)
+int record_sort(const void *r1, const void *r2)
 {
-	const sdp_record_t *rec1 = (const sdp_record_t *) r1;
-	const sdp_record_t *rec2 = (const sdp_record_t *) r2;
+	const sdp_record_t *rec1 = r1;
+	const sdp_record_t *rec2 = r2;
 
 	if (!rec1 || !rec2) {
 		error("NULL RECORD LIST FATAL");
@@ -69,8 +53,8 @@ static int record_sort(const void *r1, const void *r2)
 
 static int access_sort(const void *r1, const void *r2)
 {
-	const sdp_access_t *rec1 = (const sdp_access_t *) r1;
-	const sdp_access_t *rec2 = (const sdp_access_t *) r2;
+	const sdp_access_t *rec1 = r1;
+	const sdp_access_t *rec2 = r2;
 
 	if (!rec1 || !rec2) {
 		error("NULL RECORD LIST FATAL");
@@ -88,10 +72,13 @@ static void access_free(void *p)
 /*
  * Reset the service repository by deleting its contents
  */
-void sdp_svcdb_reset()
+void sdp_svcdb_reset(void)
 {
 	sdp_list_free(service_db, (sdp_free_func_t) sdp_record_free);
+	service_db = NULL;
+
 	sdp_list_free(access_db, access_free);
+	access_db = NULL;
 }
 
 typedef struct _indexed {
@@ -109,7 +96,7 @@ void sdp_svcdb_collect_all(int sock)
 	sdp_list_t *p, *q;
 
 	for (p = socket_index, q = 0; p; ) {
-		sdp_indexed_t *item = (sdp_indexed_t *) p->data;
+		sdp_indexed_t *item = p->data;
 		if (item->sock == sock) {
 			sdp_list_t *next = p->next;
 			sdp_record_remove(item->record->handle);
@@ -135,7 +122,7 @@ void sdp_svcdb_collect(sdp_record_t *rec)
 	sdp_list_t *p, *q;
 
 	for (p = socket_index, q = 0; p; q = p, p = p->next) {
-		sdp_indexed_t *item = (sdp_indexed_t *) p->data;
+		sdp_indexed_t *item = p->data;
 		if (rec == item->record) {
 			free(item);
 			if (q)
@@ -150,14 +137,20 @@ void sdp_svcdb_collect(sdp_record_t *rec)
 
 static int compare_indices(const void *i1, const void *i2)
 {
-	const sdp_indexed_t *s1 = (const sdp_indexed_t *) i1;
-	const sdp_indexed_t *s2 = (const sdp_indexed_t *) i2;
+	const sdp_indexed_t *s1 = i1;
+	const sdp_indexed_t *s2 = i2;
 	return s1->sock - s2->sock;
 }
 
 void sdp_svcdb_set_collectable(sdp_record_t *record, int sock)
 {
 	sdp_indexed_t *item = malloc(sizeof(sdp_indexed_t));
+
+	if (!item) {
+		SDPDBG("No memory");
+		return;
+	}
+
 	item->sock = sock;
 	item->record = record;
 	socket_index = sdp_list_insert_sorted(socket_index, item, compare_indices);
@@ -227,7 +220,7 @@ sdp_record_t *sdp_record_find(uint32_t handle)
 		return 0;
 	}
 
-	return (sdp_record_t *) p->data;
+	return p->data;
 }
 
 /*
@@ -244,18 +237,18 @@ int sdp_record_remove(uint32_t handle)
 		return -1;
 	}
 
-	r = (sdp_record_t *) p->data;
+	r = p->data;
 	if (r)
 		service_db = sdp_list_remove(service_db, r);
 
 	p = access_locate(handle);
-	if (p) {
-		a = (sdp_access_t *) p->data;
-		if (a) {
-			access_db = sdp_list_remove(access_db, a);
-			access_free(a);
-		}
-	}
+	if (p == NULL || p->data == NULL)
+		return 0;
+
+	a = p->data;
+
+	access_db = sdp_list_remove(access_db, a);
+	access_free(a);
 
 	return 0;
 }
@@ -268,11 +261,6 @@ sdp_list_t *sdp_get_record_list(void)
 	return service_db;
 }
 
-sdp_list_t *sdp_get_access_list(void)
-{
-	return access_db;
-}
-
 int sdp_check_access(uint32_t handle, bdaddr_t *device)
 {
 	sdp_list_t *p = access_locate(handle);
@@ -281,7 +269,7 @@ int sdp_check_access(uint32_t handle, bdaddr_t *device)
 	if (!p)
 		return 1;
 
-	a = (sdp_access_t *) p->data;
+	a = p->data;
 	if (!a)
 		return 1;
 
