@@ -29,7 +29,7 @@ void set_menu_prompt(const char *name, const char *id)
 {
 	char *prompt;
 
-	prompt = g_strdup_printf("[%s%s%s]# ", name,
+	prompt = g_strdup_printf("[%s%s%s]> ", name,
 					id ? ": Target = " : "", id ? id : "");
 	bt_shell_set_prompt(prompt, COLOR_BLUE);
 	g_free(prompt);
